// Pivoting platform adapted from Dan Shiffman's rectangular box class.
class Platform  {

  Body body;
  float w;
  float h;
  float x;
  float y;

  // Constructor
  Platform(float _x, float _y) {    
    x = _x;
    y = _y;
    w = width * .65;
    h = width * .65;
    // Add the box to the box2d world 
    makeBody(new Vec2(x,y),w,h);
  }

  // This function removes the particle from the box2d world
  void killBody() {
    box2d.destroyBody(body);
  }

  // Drawing the box
  void display() {
    // We look at each body and get its screen position
    Vec2 pos = box2d.getScreenPos(body);
    // Get its angle of rotation
    float a = body.getAngle();

    rectMode(CENTER);
    pushMatrix();
    translate(pos.x,pos.y);
    rotate(-a);
    fill(150);
    noStroke();
    rect(0,0,w,h);
    popMatrix();
  }

  // This function adds the rectangle to the box2d world
  void makeBody(Vec2 center, float w_, float h_) {

    // Define a polygon (this is what we use for a rectangle)
    PolygonDef sd = new PolygonDef();
    float box2dW = box2d.scaleScreenToWorld(w_/2);
    float box2dH = box2d.scaleScreenToWorld(h_/2);
    sd.setAsBox(box2dW, box2dH);

    // Parameters that affect physics
    sd.density = 1.0f;
    sd.friction = 0.8f;
    sd.restitution = 0.005f;

    // Define the body and make it from the shape
    BodyDef bd = new BodyDef();
    bd.position.set(box2d.screenToWorld(center));

    body = box2d.createBody(bd);
    body.createShape(sd);
    body.setMassFromShapes();

    // Give it some initial random velocity
    //body.setAngularVelocity(random(-5,5));
  }
}



