import pbox2d.*;
import org.jbox2d.collision.shapes.*;
import org.jbox2d.common.*;
import org.jbox2d.dynamics.*;
import org.jbox2d.dynamics.joints.*;
import controlP5.*;

// Weight of Your Words
// Eric Mika, 2010
// Nature of Code @ ITP

// Docs on Dan's Box2D wrapper:
// http://code.google.com/p/pbox2d/

ControlP5 controlP5;
Textfield textField;
String currentWord = "";
String textFieldValue = "";
GoogleScraper googleScraper;
PBox2D box2d;
PFont din;
PFont dinBlack;
Platform platform;
Boundary boundary;
RevoluteJoint joint;
RevoluteJointDef jointDef;
ArrayList words;
PImage logo;
PImage vignette;
Cloud textCloud;
Cloud scoreCloud;
boolean clearOnKey;
boolean repeatingYourself;
PVector mouseVector;
ArrayList usedWords;
Help help;

Long weightScore;
Long weightScoreMax;
// int wordScore;
// int wordScoreMax;


void setup() {
  size(1024, 768);
  background(255);
  fill(0);
  noStroke();
  smooth();

  // Prep the font.
  din = loadFont("DIN-MediumAlternate-20.vlw"); 
  dinBlack = loadFont("DIN-Black-20.vlw");
  textFont(din, 20);

  // Set up the text field.
  controlP5 = new ControlP5(this);
  textField = controlP5.addTextfield("textAdded",100,160,200,20);
  textField.setFocus(true);
  textField.keepFocus(true);  
  textField.setColorBackground(255);
  textField.setColorForeground(0);
  textField.setColorActive(0);
  textField.setColorValue(0);  
  textField.setCaptionLabel(""); 
  clearOnKey = false;
  
  // Initialize box2d physics and create the world.
  box2d = new PBox2D(this);
  box2d.createWorld();
  box2d.setGravity(0, -20);

  // Store the words.
  words = new ArrayList();
  
  // Keep track of used words.
  usedWords = new ArrayList();

  // Set up the Google Scraper. It takes a search term and returns the number of results.
  googleScraper = new GoogleScraper(this);

  // Add the platform.
  platform = new Platform(width / 2, 0);

  // Add the pivot joist.
  boundary = new Boundary(width / 2,  height - 75, 60, 160);

  // Set up the joint, with lots of friction.
  jointDef = new RevoluteJointDef();
  jointDef.body1 = platform.body;
  jointDef.body2 = boundary.b;
  jointDef.localAnchor2 = box2d.screenToWorld(width / 2, 600);
  jointDef.lowerAngle = -0.2f * PI;
  jointDef.upperAngle = 0.2f * PI;
  //jointDef.enableLimit = true;
  jointDef.motorSpeed = 0.0f;
  jointDef.maxMotorTorque = 1000000.0f;
  jointDef.enableMotor = true;
  joint = (RevoluteJoint) box2d.world.createJoint(jointDef);
  
  // Set up the text cloud.
  textCloud = new Cloud(new PVector(width / 2, 40), 220, 75, 100);
  logo = loadImage("logo.png");

  mouseVector = new PVector(mouseX, mouseY);
  
  vignette = loadImage("vignette.png");
  
  // Set up the score cloud.
  weightScore = 0L;
  weightScoreMax = 0L;
  scoreCloud = new Cloud(new PVector(30, 190), 200, 70, 80);
  
  // Set up the help
  help = new Help(); 
}


void draw() {
  background(255);
  
  // Advance the physics simulation.
  box2d.step();

  // Keep tabs on the Google Scraper.
  
  // Must be fetching it.
  if(googleScraper.fetching) {
    textField.setColorValue(color(180, 0, 0));    
    textField.setText("FETCHING ");
  }
  
  if(repeatingYourself) {
    textField.setColorValue(color(180, 0, 0));    
    textField.setText("YOU ALREADY SAID THAT ");
    repeatingYourself = false;
    clearOnKey = true;
  }
  
  if(googleScraper.timedOut) {
    textField.setColorValue(color(180, 0, 0));    
    textField.setText("THE SEARCH TIMED OUT ");
    googleScraper.timedOut = false;
    clearOnKey = true;
  }
  
  if(googleScraper.noResults) {
    textField.setColorValue(color(180, 0, 0));    
    textField.setText("NO RESULTS ");
    googleScraper.noResults = false;
    clearOnKey = true;
  }  
  
  // Must have received it.
  if(googleScraper.success)  {
    googleScraper.success = false;
    
    // must have the count, let's make the word
    println("adding word");
    textField.setText(nfc((float)googleScraper.resultCount, 0) + " RESULTS ");
    
    // Prepare to clear the text field as soon as we start typing.
    clearOnKey = true;
    words.add(new Word(googleScraper.latestWord, googleScraper.resultCount, textCloud.loc.x + 100, textCloud.loc.y + 50));
    currentWord = "";
  }
  

  // Display all the words.
  weightScore = 0L;
  
  for (int i = 0; i < words.size(); i++) {
    Word word = (Word) words.get(i);
    word.display();
    
    weightScore += word.resultCount;
  }
  
  if(weightScore > weightScoreMax) {
    weightScoreMax = weightScore;
  }
  
  // Clean up departed words.
  for (int i = words.size()-1; i >= 0; i--) {
    Word word = (Word) words.get(i);
    if (word.done()) {
      println("Removing word \"" + word.theWord + "\".");
      words.remove(i);
    }
  }

  // Show the see-saw cube.
  platform.display();
  
  // Don't bother showing the platform's attachment point.
  // boundary.display();
  
  
  // Make the text cloud follow the mouse (but only on the X axis).
  // This is just crude easing... not sure why the bother with vectors here.
  // (Particularly when only one axis is involved.) Maybe to save time in the future.
  // But any future time saved has been eclipsed by present time wasted
  // writing this comment. So it goes.
  mouseVector.x = mouseX;
  mouseVector.y = mouseY;

  mouseVector.x -= 125; // cloud offset
  mouseVector.sub(textCloud.loc);
  mouseVector.mult(.02);
  mouseVector.y = 0;
  textCloud.loc.add(mouseVector);
  
  // Show the text cloud.
  textCloud.tick();
  textField.setPosition(textCloud.loc.x + 10, textCloud.loc.y + 47);
  image(logo, textCloud.loc.x + 10, textCloud.loc.y + 17);

  // Show the score cloud.  
  scoreCloud.tick();
  
  image(vignette, 0, 0);
  
  pushMatrix();
  translate(scoreCloud.loc.x, scoreCloud.loc.y);
  rotate(-.1);
  textAlign(LEFT);
  fill(255);
  text("Weight", 5, 22);
  text("Now: " + nfstring(Long.toString(weightScore)), 5, 46);
  fill(180, 0, 0);
  text("Max: " + nfstring(Long.toString(weightScoreMax)), 5, 68);
  popMatrix();
  
  help.tick();
  
}

// Fires when the text field text is entered.
public void textAdded(String theText) {


  // Only submit if we're done with the last word
  if(!googleScraper.fetching) {
    
    repeatingYourself = false;

    // Only pass it to google if we've already used it
    for(int i = 0; i < usedWords.size(); i++) {
      String usedWord = (String) usedWords.get(i);
      
      println("used: " + usedWord);
      
      if(theText.equals(usedWord)) {
        repeatingYourself = true;
        break;
      }
    }

    // All clear...
    if(!repeatingYourself) {
      currentWord = theText;
      
      // Store it for future reference
      usedWords.add(currentWord);

      // find out the word weight
      googleScraper = new GoogleScraper(this);
      googleScraper.getCount(currentWord);
    }
  }
  else {
    println("slow down!");
  }
}

// Move the text cloud.
void keyPressed() {
  
  if(help.showHelp) help.showHelp = false;
  
  // Clear the result text, if we need to.
  if(clearOnKey) {
    textField.setColorValue(0);
    
    if((keyCode > 32) && (keyCode < 128)) {
      textField.setText(key + "");
    }
    else {
      textField.setText("");
    }
    
    clearOnKey = false;
  }
  
  // Use the mouse instead.
  
  // if(keyCode == LEFT) {
  //   textCloud.loc.x -= 1;
  // }
  // 
  // if(keyCode == RIGHT) {
  //   textCloud.loc.x += 1;
  // }  
  
  // if(keyCode == UP) {
  //   textCloud.loc.y -= 1;
  // }
  // 
  // if(keyCode == DOWN) {
  //   textCloud.loc.y += 1;
  // }  
}

// Like NF, adds commas to numbers, but works on Strings so we
// can handle Longs (which we need for the score).
String nfstring(String number) {
  String niceNumber = "";
  
  for(int i = number.length() - 1; i >= 0; i--) {
    char thisChar = number.charAt(i);
    niceNumber = thisChar + niceNumber;
    
    if(((number.length() - i) % 3 == 0) && (i > 0)) {
      niceNumber = "," + niceNumber;
    }
  }
  
  return niceNumber;
}

// Extend core's Map function to the Long datatype.
long mapLong(long x, long in_min, long in_max, long out_min, long out_max)  { 
  return (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min; 
}

long constrainLong(long value, long min_value, long max_value) {
  if(value > max_value) return max_value;
  if(value < min_value) return min_value;
  return value;
}
