// Adapted from Dan Shiffman's rectangular box class.
class Word  {
  Body body;
  float x;
  float y;
  float w;
  float h;
  String theWord;
  float wordWidth;
  float xPadding;
  float yPadding;
  Long resultCount;
  int alpha = 0;
  int targetColor = 0; // set by weight
  
  // Constructor
  Word(String _theWord, Long _resultCount, float _x, float _y) {
    theWord = _theWord.toUpperCase();
    resultCount = _resultCount;    
    wordWidth = textWidth(theWord);
    xPadding = 6;
    yPadding = 6;
    
    x = _x;
    y = _y;
    w = wordWidth + (xPadding * 2);
    h = 20 + (yPadding * 2);
    // Add the box to the box2d world
    makeBody(new Vec2(x,y),w,h);
  }

  // This function removes the particle from the box2d world
  void killBody() {
    box2d.destroyBody(body);
  }

  // Is the particle ready for deletion?
  boolean done() {
    // Let's find the screen position of the particle
    Vec2 pos = box2d.getScreenPos(body);
    // Is it off the bottom of the screen?
    if (pos.y > (height + 500)) {
      killBody();
      return true;
    }
    return false;
  }

  // Drawing the box
  void display() {
    // We look at each body and get its screen position
    Vec2 pos = box2d.getScreenPos(body);
    // Get its angle of rotation
    float a = body.getAngle();
    
    alpha += 3;

    rectMode(CENTER);
    pushMatrix();
    translate(pos.x,pos.y);
    rotate(-a);
    fill(targetColor, alpha);
    rect(0,0,w,h);
    fill(255, alpha);
    textAlign(CENTER, CENTER);
    text(theWord, 0, 0);    
    popMatrix();
  }

  // This function adds the rectangle to the box2d world
  void makeBody(Vec2 center, float w_, float h_) {

    // Define a polygon (this is what we use for a rectangle)
    PolygonDef sd = new PolygonDef();
    float box2dW = box2d.scaleScreenToWorld(w_/2);
    float box2dH = box2d.scaleScreenToWorld(h_/2);
    sd.setAsBox(box2dW, box2dH);

    // Parameters that affect physics
    // Set density based on score.
    
    // A billion results represents the max weight
    sd.density = (float)constrainLong(mapLong(resultCount, 0, 1200000000L, 1L, 70L), 0, 70);

    // Darker boxes are heavier. Use log to make the variations more apparent.
    println(log(sd.density));
    targetColor = round(constrain(map(log(sd.density), 0f, 4.3f, 200f, 0f), 0, 200));
    println(targetColor);
    
    sd.friction = 0.8f;
    sd.restitution = .005f;

    // Define the body and make it from the shape
    BodyDef bd = new BodyDef();
    bd.position.set(box2d.screenToWorld(center));

    body = box2d.createBody(bd);
    body.createShape(sd);
    body.setMassFromShapes();

    // Give it some initial random velocity
    //body.setLinearVelocity(new Vec2(random(-5,5),random(2,5)));
    body.setAngularVelocity(random(-0.5,0.5));

  }

}



