import processing.core.*; 
import processing.xml.*; 

import pbox2d.*; 
import org.jbox2d.collision.shapes.*; 
import org.jbox2d.common.*; 
import org.jbox2d.dynamics.*; 
import org.jbox2d.dynamics.joints.*; 
import controlP5.*; 
import processing.net.*; 

import java.applet.*; 
import java.awt.*; 
import java.awt.image.*; 
import java.awt.event.*; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class weight_of_your_words extends PApplet {








// Weight of Your Words
// Eric Mika, 2010
// Nature of Code @ ITP

// Docs on Dan's Box2D wrapper:
// http://code.google.com/p/pbox2d/

ControlP5 controlP5;
Textfield textField;
String currentWord = "";
String textFieldValue = "";
GoogleScraper googleScraper;
PBox2D box2d;
PFont din;
PFont dinBlack;
Platform platform;
Boundary boundary;
RevoluteJoint joint;
RevoluteJointDef jointDef;
ArrayList words;
PImage logo;
PImage vignette;
Cloud textCloud;
Cloud scoreCloud;
boolean clearOnKey;
boolean repeatingYourself;
PVector mouseVector;
ArrayList usedWords;
Help help;

Long weightScore;
Long weightScoreMax;
// int wordScore;
// int wordScoreMax;


public void setup() {
  size(1024, 768);
  background(255);
  fill(0);
  noStroke();
  smooth();

  // Prep the font.
  din = loadFont("DIN-MediumAlternate-20.vlw"); 
  dinBlack = loadFont("DIN-Black-20.vlw");
  textFont(din, 20);

  // Set up the text field.
  controlP5 = new ControlP5(this);
  textField = controlP5.addTextfield("textAdded",100,160,200,20);
  textField.setFocus(true);
  textField.keepFocus(true);  
  textField.setColorBackground(255);
  textField.setColorForeground(0);
  textField.setColorActive(0);
  textField.setColorValue(0);  
  textField.setCaptionLabel(""); 
  clearOnKey = false;
  
  // Initialize box2d physics and create the world.
  box2d = new PBox2D(this);
  box2d.createWorld();
  box2d.setGravity(0, -20);

  // Store the words.
  words = new ArrayList();
  
  // Keep track of used words.
  usedWords = new ArrayList();

  // Set up the Google Scraper. It takes a search term and returns the number of results.
  googleScraper = new GoogleScraper(this);

  // Add the platform.
  platform = new Platform(width / 2, 0);

  // Add the pivot joist.
  boundary = new Boundary(width / 2,  height - 75, 60, 160);

  // Set up the joint, with lots of friction.
  jointDef = new RevoluteJointDef();
  jointDef.body1 = platform.body;
  jointDef.body2 = boundary.b;
  jointDef.localAnchor2 = box2d.screenToWorld(width / 2, 600);
  jointDef.lowerAngle = -0.2f * PI;
  jointDef.upperAngle = 0.2f * PI;
  //jointDef.enableLimit = true;
  jointDef.motorSpeed = 0.0f;
  jointDef.maxMotorTorque = 1000000.0f;
  jointDef.enableMotor = true;
  joint = (RevoluteJoint) box2d.world.createJoint(jointDef);
  
  // Set up the text cloud.
  textCloud = new Cloud(new PVector(width / 2, 40), 220, 75, 100);
  logo = loadImage("logo.png");

  mouseVector = new PVector(mouseX, mouseY);
  
  vignette = loadImage("vignette.png");
  
  // Set up the score cloud.
  weightScore = 0L;
  weightScoreMax = 0L;
  scoreCloud = new Cloud(new PVector(30, 190), 200, 70, 80);
  
  // Set up the help
  help = new Help(); 
}


public void draw() {
  background(255);
  
  // Advance the physics simulation.
  box2d.step();

  // Keep tabs on the Google Scraper.
  
  // Must be fetching it.
  if(googleScraper.fetching) {
    textField.setColorValue(color(180, 0, 0));    
    textField.setText("FETCHING ");
  }
  
  if(repeatingYourself) {
    textField.setColorValue(color(180, 0, 0));    
    textField.setText("YOU ALREADY SAID THAT ");
    repeatingYourself = false;
    clearOnKey = true;
  }
  
  if(googleScraper.timedOut) {
    textField.setColorValue(color(180, 0, 0));    
    textField.setText("THE SEARCH TIMED OUT ");
    googleScraper.timedOut = false;
    clearOnKey = true;
  }
  
  if(googleScraper.noResults) {
    textField.setColorValue(color(180, 0, 0));    
    textField.setText("NO RESULTS ");
    googleScraper.noResults = false;
    clearOnKey = true;
  }  
  
  // Must have received it.
  if(googleScraper.success)  {
    googleScraper.success = false;
    
    // must have the count, let's make the word
    println("adding word");
    textField.setText(nfc((float)googleScraper.resultCount, 0) + " RESULTS ");
    
    // Prepare to clear the text field as soon as we start typing.
    clearOnKey = true;
    words.add(new Word(googleScraper.latestWord, googleScraper.resultCount, textCloud.loc.x + 100, textCloud.loc.y + 50));
    currentWord = "";
  }
  

  // Display all the words.
  weightScore = 0L;
  
  for (int i = 0; i < words.size(); i++) {
    Word word = (Word) words.get(i);
    word.display();
    
    weightScore += word.resultCount;
  }
  
  if(weightScore > weightScoreMax) {
    weightScoreMax = weightScore;
  }
  
  // Clean up departed words.
  for (int i = words.size()-1; i >= 0; i--) {
    Word word = (Word) words.get(i);
    if (word.done()) {
      println("Removing word \"" + word.theWord + "\".");
      words.remove(i);
    }
  }

  // Show the see-saw cube.
  platform.display();
  
  // Don't bother showing the platform's attachment point.
  // boundary.display();
  
  
  // Make the text cloud follow the mouse (but only on the X axis).
  // This is just crude easing... not sure why the bother with vectors here.
  // (Particularly when only one axis is involved.) Maybe to save time in the future.
  // But any future time saved has been eclipsed by present time wasted
  // writing this comment. So it goes.
  mouseVector.x = mouseX;
  mouseVector.y = mouseY;

  mouseVector.x -= 125; // cloud offset
  mouseVector.sub(textCloud.loc);
  mouseVector.mult(.02f);
  mouseVector.y = 0;
  textCloud.loc.add(mouseVector);
  
  // Show the text cloud.
  textCloud.tick();
  textField.setPosition(textCloud.loc.x + 10, textCloud.loc.y + 47);
  image(logo, textCloud.loc.x + 10, textCloud.loc.y + 17);

  // Show the score cloud.  
  scoreCloud.tick();
  
  image(vignette, 0, 0);
  
  pushMatrix();
  translate(scoreCloud.loc.x, scoreCloud.loc.y);
  rotate(-.1f);
  textAlign(LEFT);
  fill(255);
  text("Weight", 5, 22);
  text("Now: " + nfstring(Long.toString(weightScore)), 5, 46);
  fill(180, 0, 0);
  text("Max: " + nfstring(Long.toString(weightScoreMax)), 5, 68);
  popMatrix();
  
  help.tick();
  
}

// Fires when the text field text is entered.
public void textAdded(String theText) {


  // Only submit if we're done with the last word
  if(!googleScraper.fetching) {
    
    repeatingYourself = false;

    // Only pass it to google if we've already used it
    for(int i = 0; i < usedWords.size(); i++) {
      String usedWord = (String) usedWords.get(i);
      
      println("used: " + usedWord);
      
      if(theText.equals(usedWord)) {
        repeatingYourself = true;
        break;
      }
    }

    // All clear...
    if(!repeatingYourself) {
      currentWord = theText;
      
      // Store it for future reference
      usedWords.add(currentWord);

      // find out the word weight
      googleScraper = new GoogleScraper(this);
      googleScraper.getCount(currentWord);
    }
  }
  else {
    println("slow down!");
  }
}

// Move the text cloud.
public void keyPressed() {
  
  if(help.showHelp) help.showHelp = false;
  
  // Clear the result text, if we need to.
  if(clearOnKey) {
    textField.setColorValue(0);
    
    if((keyCode > 32) && (keyCode < 128)) {
      textField.setText(key + "");
    }
    else {
      textField.setText("");
    }
    
    clearOnKey = false;
  }
  
  // Use the mouse instead.
  
  // if(keyCode == LEFT) {
  //   textCloud.loc.x -= 1;
  // }
  // 
  // if(keyCode == RIGHT) {
  //   textCloud.loc.x += 1;
  // }  
  
  // if(keyCode == UP) {
  //   textCloud.loc.y -= 1;
  // }
  // 
  // if(keyCode == DOWN) {
  //   textCloud.loc.y += 1;
  // }  
}

// Like NF, adds commas to numbers, but works on Strings so we
// can handle Longs (which we need for the score).
public String nfstring(String number) {
  String niceNumber = "";
  
  for(int i = number.length() - 1; i >= 0; i--) {
    char thisChar = number.charAt(i);
    niceNumber = thisChar + niceNumber;
    
    if(((number.length() - i) % 3 == 0) && (i > 0)) {
      niceNumber = "," + niceNumber;
    }
  }
  
  return niceNumber;
}

// Extend core's Map function to the Long datatype.
public long mapLong(long x, long in_min, long in_max, long out_min, long out_max)  { 
  return (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min; 
}

public long constrainLong(long value, long min_value, long max_value) {
  if(value > max_value) return max_value;
  if(value < min_value) return min_value;
  return value;
}
// Adapted from Dan Shiffman's fixed boundary class.
class Boundary {
  // A boundary is a simple rectangle with x,y,width,and height
  float x;
  float y;
  float w;
  float h;
  
  // But we also have to make a body for box2d to know about it
  Body b;

  Boundary(float x_,float y_, float w_, float h_) {
    x = x_;
    y = y_;
    w = w_;
    h = h_;

    // Figure out the box2d coordinates
    float box2dW = box2d.scaleScreenToWorld(w/2);
    float box2dH = box2d.scaleScreenToWorld(h/2);
    Vec2 center = new Vec2(x,y);

    // Define the polygon
    PolygonDef sd = new PolygonDef();
    sd.setAsBox(box2dW, box2dH);
    sd.density = 0;    // No density means it won't move!
    sd.friction = 0.3f;

    // Create the body
    BodyDef bd = new BodyDef();
    bd.position.set(box2d.screenToWorld(center));
    b = box2d.createBody(bd);
    b.createShape(sd);
  }

  // Draw the boundary, if it were at an angle we'd have to do something fancier
  public void display() {
    fill(0);
    stroke(0);
    rectMode(CENTER);
    rect(x,y,w,h);
  }

}
// Adapted from Dan Shiffman' Thing Class
class Cloud {
  PVector loc;
  PVector vel;
  PVector acc;
  float max_vel;
  float w, h;
  int vaporDensity;
  float theta;
  
  ArrayList vapors;
  Random gaussianRandom;
      
  
  Cloud(PVector l, float _w, float _h, int _vaporDensity) {
    w = _w;
    h = _h;
    
    acc = new PVector(0, 0);
    vel = new PVector(0, 0);
    
    max_vel = 5;
    
    theta = 0;
    
    vaporDensity = _vaporDensity;
    vapors = new ArrayList();
    
    loc = l.get();
    
    gaussianRandom = new Random();    
    
    while(vapors.size() < vaporDensity) {
      
      float tempW = (float)gaussianRandom.nextGaussian() * (w / 6);
      float tempH = (float)gaussianRandom.nextGaussian() * (h / 6);
      
      tempW = random(0, w);
      tempH = random(0, h);      
      
      vapors.add(new Vapor(new PVector(tempW, tempH)));
    }
    
    
  }

  // Main method to operate object
  public void tick() {
    update();
    render();
  }
  
  // Method to update location
  public void update() {
    vel.add(acc);
    vel.limit(max_vel);
    loc.add(vel);
    //acc.mult(0);
    theta += random(0, 0.02f);
    
    
  }
  
  // Method to display
  public void render() {

    pushMatrix();
    translate(loc.x, loc.y + sin(theta) * 4);
    for(int i = 0; i < vapors.size(); i++) {
      Vapor vapor = (Vapor) vapors.get(i);

      vapor.tick();

    }
    popMatrix();

  }
}
// Returns the number of results for a particular search query
// loadStrings() didn't cut it, Google returns 403s unless you fib the header a bit.
// Runs in its own thread so we don't clutter up the draw loop or halt renderiung.

// Could have used the Yahoo API with much less grief, but Google was a
// better conceptual fit.



class GoogleScraper implements Runnable {
  Client client;
  String html;
  PApplet parent;
  Thread scraperThread;
  String latestWord;
  String query;
  Long resultCount;
  int startTime;
  int timeOut;
  boolean runThread;
  boolean timedOut;
  boolean noResults;
  boolean fetching;
  boolean success;
  

  GoogleScraper(PApplet _parent) {
    parent = _parent;
    client = new Client(parent, "www.google.com", 80);

    // Create the thread supplying it with the runnable object.
    scraperThread = new Thread(this);

    startTime = 0;
    timeOut = 5000; // Give up after five seconds
    timedOut = false;
    noResults = false;
    fetching = false;
    success = false;

    // Start the thread.
    runThread = true;
    scraperThread.start();
    
  }

  // One time use...
  public void getCount(String _query) {
    query = _query;
    fetching = true;

    println("Searching for \"" + query + "\" with query \"" + URLEncoder.encode(query) + "\".");
    
    // Ham up the header a bit so we don't get 403'd.
    // Encode the URL for more advanced queries. 
    client.write("GET /search?q=" + URLEncoder.encode(query) + " HTTP/1.1\n");
    client.write("Host: www.google.com\n\n");
    client.write("User-Agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1)\n\n");

    startTime = millis();
  }


  public void gotResult() {
      fetching = false;
      runThread = false;
    
      // This is kludgey... the result count is always the third bolded number.
      if(!html.equals("")) {
        String[][] matches = matchAll(html, "<b>[0-9,]+</b>");
      
        println(matches);
      
        if((matches != null) && (matches.length >= 3)) {
          String rawResult = matches[matches.length - 1][0];
          resultCount = Long.parseLong(rawResult.substring(3, rawResult.length() - 4).replaceAll(",", ""));
          println(resultCount);
          success = true;
        }
        else {
          println("No results.");
          noResults = true;
        }
      }
      else {
        println("Timed out.");
        timedOut = true;
      }
  }

  public void run() {
    // Always running.
    while(true) {
      
      if(runThread) {
      
        if ((latestWord != query) && (client.available() > 0)) {
          html += client.readString();

          // Stop looking once we're past the point of interest.
          if(html.indexOf("<h2 class=hd>") > 0) {
            latestWord = query;
            //client.stop();
            //client.clear();
            //println(html);
            gotResult();
          }
        }
      
        // Give up if it's taking too long.
        if ((startTime > 0) && ((millis() - startTime) > timeOut)) {
          latestWord = "";
          html = "";
          startTime = 0;
          gotResult();

        }
      
      }

      try {
        // Don't hog the CPU.
        scraperThread.sleep(20);
      }
      catch(InterruptedException e) {
        e.printStackTrace();
      }
    }
  }
  
}
class Help {
  boolean showHelp;
  int fader;

  Help() {
    showHelp = true;
    fader = 255;
  }
  
  public void tick() {
    if (!showHelp && (fader > 0)) {
      fader -= 5;
      println("fading");
    }
    
    if (showHelp || (fader > 0)) {
      textAlign(CENTER);

      textFont(dinBlack, 20);
      fill(180, 0, 0, fader);
      text("WEIGHT OF YOUR WORDS", width / 2, 275);

      textFont(din, 20);
      fill(0, fader);
      text("SEED THE CLOUD WITH A SEARCH TERM, HIT ENTER... AND DOWN IT GOES. \n\n" +
         "THE MORE RESULTS A TERM HAS, THE HEAVIER IT HITS, AND THE FURTHER YOU'LL TILT.\n\n" +
         "THE CLOUD FOLLOWS YOUR MOUSE. HEAVIER TERMS APPEAR DARKER.\n\n" +
         "HOW MUCH WEIGHT CAN YOU STACK ON THE LEDGE BELOW?", width / 2, height / 2);    
    }
  }

}
// Pivoting platform adapted from Dan Shiffman's rectangular box class.
class Platform  {

  Body body;
  float w;
  float h;
  float x;
  float y;

  // Constructor
  Platform(float _x, float _y) {    
    x = _x;
    y = _y;
    w = width * .65f;
    h = width * .65f;
    // Add the box to the box2d world 
    makeBody(new Vec2(x,y),w,h);
  }

  // This function removes the particle from the box2d world
  public void killBody() {
    box2d.destroyBody(body);
  }

  // Drawing the box
  public void display() {
    // We look at each body and get its screen position
    Vec2 pos = box2d.getScreenPos(body);
    // Get its angle of rotation
    float a = body.getAngle();

    rectMode(CENTER);
    pushMatrix();
    translate(pos.x,pos.y);
    rotate(-a);
    fill(150);
    noStroke();
    rect(0,0,w,h);
    popMatrix();
  }

  // This function adds the rectangle to the box2d world
  public void makeBody(Vec2 center, float w_, float h_) {

    // Define a polygon (this is what we use for a rectangle)
    PolygonDef sd = new PolygonDef();
    float box2dW = box2d.scaleScreenToWorld(w_/2);
    float box2dH = box2d.scaleScreenToWorld(h_/2);
    sd.setAsBox(box2dW, box2dH);

    // Parameters that affect physics
    sd.density = 1.0f;
    sd.friction = 0.8f;
    sd.restitution = 0.005f;

    // Define the body and make it from the shape
    BodyDef bd = new BodyDef();
    bd.position.set(box2d.screenToWorld(center));

    body = box2d.createBody(bd);
    body.createShape(sd);
    body.setMassFromShapes();

    // Give it some initial random velocity
    //body.setAngularVelocity(random(-5,5));
  }
}



// Adapted from Dan Shiffman' Thing Class
class Vapor {
  PVector loc;

  
  ArrayList vapor;

  float theta;
    
  Vapor(PVector l) {
    loc = l.get();
    theta = random(0, 10);
  }

  // Main method to operate object
  public void tick() {
    update();
    render();
  }
  
  // Method to update location
  public void update() {
    theta += random(0, 0.05f);
  }
  
  // Method to display
  public void render() {
    fill(220);
    ellipseMode(CENTER);
    ellipse(loc.x, loc.y, 5 * (sin(theta) + 10), 5 * (sin(theta) + 10));


  }
}
// Adapted from Dan Shiffman's rectangular box class.
class Word  {
  Body body;
  float x;
  float y;
  float w;
  float h;
  String theWord;
  float wordWidth;
  float xPadding;
  float yPadding;
  Long resultCount;
  int alpha = 0;
  int targetColor = 0; // set by weight
  
  // Constructor
  Word(String _theWord, Long _resultCount, float _x, float _y) {
    theWord = _theWord.toUpperCase();
    resultCount = _resultCount;    
    wordWidth = textWidth(theWord);
    xPadding = 6;
    yPadding = 6;
    
    x = _x;
    y = _y;
    w = wordWidth + (xPadding * 2);
    h = 20 + (yPadding * 2);
    // Add the box to the box2d world
    makeBody(new Vec2(x,y),w,h);
  }

  // This function removes the particle from the box2d world
  public void killBody() {
    box2d.destroyBody(body);
  }

  // Is the particle ready for deletion?
  public boolean done() {
    // Let's find the screen position of the particle
    Vec2 pos = box2d.getScreenPos(body);
    // Is it off the bottom of the screen?
    if (pos.y > (height + 500)) {
      killBody();
      return true;
    }
    return false;
  }

  // Drawing the box
  public void display() {
    // We look at each body and get its screen position
    Vec2 pos = box2d.getScreenPos(body);
    // Get its angle of rotation
    float a = body.getAngle();
    
    alpha += 3;

    rectMode(CENTER);
    pushMatrix();
    translate(pos.x,pos.y);
    rotate(-a);
    fill(targetColor, alpha);
    rect(0,0,w,h);
    fill(255, alpha);
    textAlign(CENTER, CENTER);
    text(theWord, 0, 0);    
    popMatrix();
  }

  // This function adds the rectangle to the box2d world
  public void makeBody(Vec2 center, float w_, float h_) {

    // Define a polygon (this is what we use for a rectangle)
    PolygonDef sd = new PolygonDef();
    float box2dW = box2d.scaleScreenToWorld(w_/2);
    float box2dH = box2d.scaleScreenToWorld(h_/2);
    sd.setAsBox(box2dW, box2dH);

    // Parameters that affect physics
    // Set density based on score.
    
    // A billion results represents the max weight
    sd.density = (float)constrainLong(mapLong(resultCount, 0, 1200000000L, 1L, 70L), 0, 70);

    // Darker boxes are heavier. Use log to make the variations more apparent.
    println(log(sd.density));
    targetColor = round(constrain(map(log(sd.density), 0f, 4.3f, 200f, 0f), 0, 200));
    println(targetColor);
    
    sd.friction = 0.8f;
    sd.restitution = .005f;

    // Define the body and make it from the shape
    BodyDef bd = new BodyDef();
    bd.position.set(box2d.screenToWorld(center));

    body = box2d.createBody(bd);
    body.createShape(sd);
    body.setMassFromShapes();

    // Give it some initial random velocity
    //body.setLinearVelocity(new Vec2(random(-5,5),random(2,5)));
    body.setAngularVelocity(random(-0.5f,0.5f));

  }

}




  static public void main(String args[]) {
    PApplet.main(new String[] { "--bgcolor=#FFFFFF", "weight_of_your_words" });
  }
}
